@@ -36,7 +36,8 @@ const kpActions = {
     DATABASE_LOCKED: 'database-locked',
     DATABASE_UNLOCKED: 'database-unlocked',
     GET_DATABASE_GROUPS: 'get-database-groups',
-    CREATE_NEW_GROUP: 'create-new-group'
+    CREATE_NEW_GROUP: 'create-new-group',
+    GET_TOTP: 'get-totp'
 };
 
 const kpErrors = {
@@ -86,6 +87,27 @@ browser.storage.local.get({ 'latestKeePassXC': { 'version': '', 'lastChecked': n
     keepass.keyRing = item.keyRing;
 });
 
+const messageBuffer = {
+    buffer: [],
+
+    addMessage(msg) {
+        if (!this.buffer.includes(msg)) {
+            this.buffer.push(msg);
+        }
+    },
+
+    matchAndRemove(msg) {
+        for (let i = 0; i < this.buffer.length; ++i) {
+            if (msg.nonce && msg.nonce === keepass.incrementedNonce(this.buffer[i].nonce)) {
+                this.buffer.splice(i, 1);
+                return true;
+            }
+        }
+
+        return false;
+    }
+};
+
 keepass.sendNativeMessage = function(request, enableTimeout = false, timeoutValue) {
     return new Promise((resolve, reject) => {
         let timeout;
@@ -95,11 +117,16 @@ keepass.sendNativeMessage = function(request, enableTimeout = false, timeoutValu
         const listener = ((port, action) => {
             const handler = (msg) => {
                 if (msg && msg.action === action) {
-                    port.removeListener(handler);
-                    if (enableTimeout) {
-                        clearTimeout(timeout);
+                    // Only resolve a matching response or a notification (without nonce)
+                    if (!msg.nonce || messageBuffer.matchAndRemove(msg)) {
+                        port.removeListener(handler);
+                        if (enableTimeout) {
+                            clearTimeout(timeout);
+                        }
+
+                        resolve(msg);
+                        return;
                     }
-                    resolve(msg);
                 }
             };
             return handler;
@@ -122,6 +149,9 @@ keepass.sendNativeMessage = function(request, enableTimeout = false, timeoutValu
             }, messageTimeout);
         }
 
+        // Store the request to the buffer
+        messageBuffer.addMessage(request);
+
         // Send the request
         if (keepass.nativePort) {
             keepass.nativePort.postMessage(request);
@@ -777,6 +807,51 @@ keepass.createNewGroup = async function(tab, args = []) {
     }
 };
 
+keepass.getTotp = async function(tab, args = []) {
+    const [ uuid, oldTotp ] = args;
+    if (!keepass.compareVersion('2.6.1', keepass.currentKeePassXC, true)) {
+        return oldTotp;
+    }
+
+    const taResponse = await keepass.testAssociation(tab, [ false ]);
+    if (!taResponse || !keepass.isConnected) {
+        return;
+    }
+
+    const kpAction = kpActions.GET_TOTP;
+    const [ nonce, incrementedNonce ] = keepass.getNonces();
+
+    const messageData = {
+        action: kpAction,
+        uuid: uuid
+    };
+
+    try {
+        const request = keepass.buildRequest(kpAction, keepass.encrypt(messageData, nonce), nonce, keepass.clientID);
+        const response = await keepass.sendNativeMessage(request);
+        if (response.message && response.nonce) {
+            const res = keepass.decrypt(response.message, response.nonce);
+            if (!res) {
+                keepass.handleError(tab, kpErrors.CANNOT_DECRYPT_MESSAGE);
+                return;
+            }
+
+            const message = nacl.util.encodeUTF8(res);
+            const parsed = JSON.parse(message);
+            if (keepass.verifyResponse(parsed, incrementedNonce) && parsed.totp) {
+                keepass.updateLastUsed(keepass.databaseHash);
+                return parsed.totp;
+            }
+        } else if (response.error && response.errorCode) {
+            keepass.handleError(tab, response.errorCode, response.error);
+        }
+
+        return;
+    } catch (err) {
+        console.log('getTotp failed: ', err);
+    }
+};
+
 keepass.generateNewKeyPair = function() {
     keepass.keyPair = nacl.box.keyPair();
     //console.log(nacl.util.encodeBase64(keepass.keyPair.publicKey) + ' ' + nacl.util.encodeBase64(keepass.keyPair.secretKey));
@@ -1159,9 +1234,7 @@ keepass.reconnect = async function(tab, connectionTimeout) {
 
 keepass.updatePopup = function(iconType) {
     if (page && page.tabs.length > 0) {
-        const data = page.tabs[page.currentTabId].stack[page.tabs[page.currentTabId].stack.length - 1];
-        data.iconType = iconType;
-        browserAction.show({ 'id': page.currentTabId });
+        browserAction.updateIcon(undefined, iconType);
     }
 };
 
@@ -1169,7 +1242,8 @@ keepass.updatePopup = function(iconType) {
 keepass.updateDatabase = async function() {
     keepass.associated.value = false;
     keepass.associated.hash = null;
-    await keepass.testAssociation(null);
+    page.clearAllLogins();
+    await keepass.testAssociation(null, [ true ]);
     const configured = await keepass.isConfigured();
     keepass.updatePopup(configured ? 'normal' : 'locked');
     keepass.updateDatabaseHashToContent();
