@@ -1,8 +1,20 @@
 'use strict';
 
-const ignoreRegex = /(zip|postal).*code/i;
+const ignoreRegex = /(bank|coupon|postal|user|zip).*code|comment|author/i;
 const ignoredTypes = [ 'email', 'password', 'username' ];
 
+const acceptedOTPFields = [
+    '2fa',
+    'auth',
+    'challenge',
+    'code',
+    'idvpin',
+    'mfa',
+    'otp',
+    'token',
+    'twofactor'
+];
+
 var kpxcTOTPIcons = {};
 kpxcTOTPIcons.icons = [];
 
@@ -18,33 +30,34 @@ kpxcTOTPIcons.deleteHiddenIcons = function() {
     kpxcUI.deleteHiddenIcons(kpxcTOTPIcons.icons, 'kpxc-totp-field');
 };
 
+// Quick check for a valid TOTP field
+kpxcTOTPIcons.isAcceptedTOTPField = function(field) {
+    const id = field.getLowerCaseAttribute('id');
+    const name = field.getLowerCaseAttribute('name');
+    const autocomplete = field.getLowerCaseAttribute('autocomplete');
+    const placeholder = field.getLowerCaseAttribute('placeholder');
 
-class TOTPFieldIcon extends Icon {
-    constructor(field, databaseState = DatabaseState.DISCONNECTED, forced = false) {
-        super();
-        this.icon = null;
-        this.inputField = null;
-        this.databaseState = databaseState;
-
-        if (this.initField(field, forced)) {
-            kpxcUI.monitorIconPosition(this);
-        }
+    // Checks if the field id, name or placeholder includes some of the acceptedOTPFields but not any from ignoredTypes
+    if (autocomplete === 'one-time-code'
+        || (acceptedOTPFields.some(f => (id && id.includes(f)) || (name && name.includes(f) || placeholder && placeholder.includes(f))))
+            && !ignoredTypes.some(f => (id && id.includes(f)) || (name && name.includes(f) || placeholder && placeholder.includes(f)))) {
+        return true;
     }
-}
 
-TOTPFieldIcon.prototype.initField = function(field, forced) {
-    if (!field) {
-        return;
+    return false;
+};
+
+kpxcTOTPIcons.isValid = function(field, forced) {
+    if (!field || !kpxcTOTPIcons.isAcceptedTOTPField(field)) {
+        return false;
     }
 
     if (!forced) {
         if (ignoredTypes.some(t => t === field.type)
-            || field.offsetWidth < MINIMUM_INPUT_FIELD_WIDTH
+            || field.offsetWidth < MIN_INPUT_FIELD_OFFSET_WIDTH
             || field.size < 2
-            || (field.maxLength > 0 && (field.maxLength < 6 || field.maxLength > 8))
+            || (field.maxLength > 0 && (field.maxLength < MIN_TOTP_INPUT_LENGTH || field.maxLength > kpxcSites.expectedTOTPMaxLength()))
             || ignoredTypes.some(t => t === field.autocomplete)
-            || field.getAttribute('kpxc-totp-field') === 'true'
-            || (field.hasAttribute('kpxc-defined') && field.getAttribute('kpxc-defined') !== 'totp')
             || field.id.match(ignoreRegex)
             || field.name.match(ignoreRegex)
             || field.readOnly) {
@@ -56,8 +69,19 @@ TOTPFieldIcon.prototype.initField = function(field, forced) {
         }
     }
 
-    field.setAttribute('kpxc-totp-field', 'true');
+    return true;
+};
 
+class TOTPFieldIcon extends Icon {
+    constructor(field, databaseState = DatabaseState.DISCONNECTED, forced = false) {
+        super(field, databaseState);
+
+        this.initField(field, forced);
+        kpxcUI.monitorIconPosition(this);
+    }
+}
+
+TOTPFieldIcon.prototype.initField = function(field, forced) {
     // Observer the visibility
     if (this.observer) {
         this.observer.observe(field);
@@ -65,7 +89,6 @@ TOTPFieldIcon.prototype.initField = function(field, forced) {
 
     this.createIcon(field);
     this.inputField = field;
-    return false;
 };
 
 TOTPFieldIcon.prototype.createIcon = function(field) {
@@ -97,10 +120,10 @@ TOTPFieldIcon.prototype.createIcon = function(field) {
 
         e.preventDefault();
         await kpxc.receiveCredentialsIfNecessary();
-        kpxc.fillInFromActiveElementTOTPOnly(field);
+        kpxc.fillFromTOTP(field);
     });
 
-    kpxcUI.setIconPosition(icon, field);
+    kpxcUI.setIconPosition(icon, field, this.rtl);
     this.icon = icon;
 
     const styleSheet = document.createElement('link');
